@@ -37,19 +37,21 @@ artText_ansiRegular = """
 ██████  █████   ███████ ██      ██████  █████   █████   ██ ██  ██ █████   ██████
 ██      ██  ██       ██ ██      ██   ██ ██      ██      ██  ██ ██ ██      ██   ██
 ██      ██   ██ ███████  ██████ ██   ██ ███████ ███████ ██   ████ ███████ ██   ██
+UPI: PKScreener@APL
 """
 artText_Merlin1 = """
    _______   __   ___   ________  ______    _______    _______   _______  _____  ___    _______   _______TM 🇮🇳
-  |   __ "\ |/"| /  ") /"       )/" _  "\  /"      \  /"     "| /"     "|(\"   \|"  \  /"     "| /"      \  
-  (. |__) :)(: |/   / (:   \___/(: ( \___)|:        |(: ______)(: ______)|.\\   \    |(: ______)|:        |
-  |:  ____/ |    __/   \___  \   \/ \     |_____/   ) \/    |   \/    |  |: \.   \\  | \/    |  |_____/   )
-  (|  /     (// _  \    __/  \\  //  \ _   //      /  // ___)_  // ___)_ |.  \    \. | // ___)_  //      /
+  |   __ "\ |/"| /  ") /"       )/" _  "\  /"      \  /"     "| /"     "|(\    \|"  \  /"     "| /"      \  
+  (. |__) :)(: |/   / (:   \___/(: ( \___)|:        |(: ______)(: ______)|.\    \    |(: ______)|:        |
+  |:  ____/ |    __/   \___  \   \/ \     |_____/   ) \/    |   \/    |  |: \.   \   | \/    |  |_____/   )
+  (|  /     (// _  \    __/   \  //  \ _   //      /  // ___)_  // ___)_ |.  \    \. | // ___)_  //      /
  /|__/ \    |: | \  \  /" \   :)(:   _) \ |:  __   \ (:      "|(:      "||    \    \ |(:      "||:  __   \ 
 (_______)   (__|  \__)(_______/  \_______)|__|  \___) \_______) \_______) \___|\____\) \_______)|__|  \___)
+UPI: PKScreener@APL
 """
 artText_dos_rebel = """
  ███████████  █████   ████  █████████                                                                  TM 🇮🇳
-░░███░░░░░███░░███   ███░  ███░░░░░███  MADE IN INDIA (UPI: 8007162973@APL)
+░░███░░░░░███░░███   ███░  ███░░░░░███  MADE IN INDIA (UPI: PKScreener@APL)
  ░███    ░███ ░███  ███   ░███    ░░░   ██████  ████████   ██████   ██████  ████████    ██████  ████████
  ░██████████  ░███████    ░░█████████  ███░░███░░███░░███ ███░░███ ███░░███░░███░░███  ███░░███░░███░░███
  ░███░░░░░░   ░███░░███    ░░░░░░░░███░███ ░░░  ░███ ░░░ ░███████ ░███████  ░███ ░███ ░███████  ░███ ░░░
@@ -59,7 +61,7 @@ artText_dos_rebel = """
 """
 artText_Puffy = """
  ___    _   _  ___                                                 
-(  _`\ ( ) ( )(  _`\   MADE IN INDIA (UPI: 8007162973@APL)       TM 🇮🇳
+(  _`\ ( ) ( )(  _`\   MADE IN INDIA (UPI: PKScreener@APL)       TM 🇮🇳
 | |_) )| |/'/'| (_(_)   ___  _ __   __     __    ___     __   _ __ 
 | ,__/'| , <  `\__ \  /'___)( '__)/'__`\ /'__`\/' _ `\ /'__`\( '__)
 | |    | |\`\ ( )_) |( (___ | |  (  ___/(  ___/| ( ) |(  ___/| |   
@@ -67,14 +69,14 @@ artText_Puffy = """
 """
 artText_Rounded = """
  ______ _     _  ______                                           
-(_____ (_)   | |/ _____) MADE IN INDIA (UPI: 8007162973@APL)    TM 🇮🇳
+(_____ (_)   | |/ _____) MADE IN INDIA (UPI: PKScreener@APL)    TM 🇮🇳
  _____) )____| ( (____   ____  ____ _____ _____ ____  _____  ____ 
 |  ____/  _   _)\____ \ / ___)/ ___) ___ | ___ |  _ \| ___ |/ ___)
 | |    | |  \ \ _____) | (___| |   | ____| ____| | | | ____| |    
 |_|    |_|   \_|______/ \____)_|   |_____)_____)_| |_|_____)_|
 """
 artText_Standard = """
- ____  _  ______  MADE IN INDIA (UPI: 8007162973@APL)  🇮🇳
+ ____  _  ______  MADE IN INDIA (UPI: PKScreener@APL)  🇮🇳
 |  _ \| |/ / ___|  ___ _ __ ___  ___ _ __   ___ _ __TM
 | |_) | ' /\___ \ / __| '__/ _ \/ _ \ '_ \ / _ \ '__|
 |  __/| . \ ___) | (__| | |  __/  __/ | | |  __/ |   
@@ -82,15 +84,15 @@ artText_Standard = """
 """
 artText_Varsity = """
  _______  ___  ____   ______                                                       
-|_   __ \|_  ||_  _|.' ____ \  MADE IN INDIA (UPI: 8007162973@APL)              TM 🇮🇳
+|_   __ \|_  ||_  _|.' ____ \  MADE IN INDIA (UPI: PKScreener@APL)              TM 🇮🇳
   | |__) | | |_/ /  | (___ \_| .---.  _ .--.  .---.  .---.  _ .--.  .---.  _ .--.  
   |  ___/  |  __'.   _.____`. / /'`\][ `/'`\]/ /__||/ /__||[ `.-. || / /_| |[ `/'`\] 
  _| |_    _| |  \ \_| \____) || \__.  | |    | \__.,| \__., | | | || \__., | |     
 |_____|  |____||____|\______.''.___.'[___]    '.__.' '.__.'[___||__]'.__.'[___]
 """
 artText_Doh = """
-PPPPPPPPPPPPPPPPP   KKKKKKKKK    KKKKKKK   SSSSSSSSSSSSSSS                                                                                                                                         TM 🇮🇳
-UPI:8007162973@APL  K:::::::K    K:::::K SS:::::::::::::::S
+PPPPPPPPPPPPPPPPP   KKKKKKKKK    KKKKKKK   SSSSSSSSSSSSSSS                                                                                                            UPI: PKScreener@APL        TM 🇮🇳
+UPI:PKScreener@APL  K:::::::K    K:::::K SS:::::::::::::::S
 P::::::PPPPPP:::::P K:::::::K    K:::::KS:::::SSSSSS::::::S
 PP:::::P     P:::::PK:::::::K   K::::::KS:::::S     SSSSSSS
   P::::P     P:::::P K::::::K  K:::::K  S:::::S                ccccccccccccccccrrrrr   rrrrrrrrr       eeeeeeeeeeee        eeeeeeeeeeee    nnnn  nnnnnnnn        eeeeeeeeeeee    rrrrr   rrrrrrrrr
@@ -109,7 +111,7 @@ PPPPPPPPPP            KKKKKKK    KKKKKKK SSSSSSSSSSSSSSS       cccccccccccccccc
 artText_Collosol = """
 8888888b.  888    d8P   .d8888b.                                                            TM 🇮🇳
 888   Y88b 888   d8P   d88P  Y88b
-888    888 888  d8P    Y88b.        MADE IN INDIA (UPI: 8007162973@APL)
+888    888 888  d8P    Y88b.        MADE IN INDIA (UPI: PKScreener@APL)
 888   d88P 888d88K      "Y888b.    .d8888b 888d888  .d88b.   .d88b.  88888b.   .d88b.  888d888
 8888888P"  8888888b        "Y88b. d88P"    888P"   d8P  Y8b d8P  Y8b 888 "88b d8P  Y8b 888P"
 888        888  Y88b         "888 888      888     88888888 88888888 888  888 88888888 888
@@ -119,7 +121,7 @@ artText_Collosol = """
 
 artText_Roman="""
 ooooooooo.   oooo    oooo  .oooooo..o                                                                    TM 🇮🇳
-`888   `Y88. `888   .8P'  d8P'    `Y8   MADE IN INDIA (UPI: 8007162973@APL)
+`888   `Y88. `888   .8P'  d8P'    `Y8   MADE IN INDIA (UPI: PKScreener@APL)
  888   .d88'  888  d8'    Y88bo.       .ooooo.  oooo d8b  .ooooo.   .ooooo.  ooo. .oo.    .ooooo.  oooo d8b
  888ooo88P'   88888[       `"Y8888o.  d88' `"Y8 `888""8P d88' `88b d88' `88b `888P"Y88b  d88' `88b `888""8P
  888          888`88b.         `"Y88b 888        888     888ooo888 888ooo888  888   888  888ooo888  888
@@ -128,7 +130,7 @@ o888o        o888o  o888o 8""88888P'  `Y8bod8P' d888b    `Y8bod8P' `Y8bod8P' o88
 """
 artText_Electronic="""
  ▄▄▄▄▄▄▄▄▄▄▄ ▄    ▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄        ▄ ▄▄▄▄▄▄▄▄▄▄▄ ▄▄▄▄▄▄▄▄▄▄TM 🇮🇳
-▐░░░░░░░░░░░▐░▌  ▐░▐░░░░░░░░░░░▐ 8007162973▐@APL░░░░░░░▐░░░░MADE IN▐INDIA░░░░░░▐░░▌      ▐░▐░░░░░░░░░░░▐░░░░░░░░░░░▌
+▐░░░░░░░░░░░▐░▌  ▐░▐░░░░░░░░░░░▐ PKScreener▐@APL░░░░░░░▐░░░░MADE IN▐INDIA░░░░░░▐░░▌      ▐░▐░░░░░░░░░░░▐░░░░░░░░░░░▌
 ▐░█▀▀▀▀▀▀▀█░▐░▌ ▐░▌▐░█▀▀▀▀▀▀▀▀▀▐░█▀▀▀▀▀▀▀▀▀▐░█▀▀▀▀▀▀▀█░▐░█▀▀▀▀▀▀▀▀▀▐░█▀▀▀▀▀▀▀▀▀▐░▌░▌     ▐░▐░█▀▀▀▀▀▀▀▀▀▐░█▀▀▀▀▀▀▀█░▌
 ▐░▌       ▐░▐░▌▐░▌ ▐░▌         ▐░▌         ▐░▌       ▐░▐░▌         ▐░▌         ▐░▌▐░▌    ▐░▐░▌         ▐░▌       ▐░▌
 ▐░█▄▄▄▄▄▄▄█░▐░▌░▌  ▐░█▄▄▄▄▄▄▄▄▄▐░▌         ▐░█▄▄▄▄▄▄▄█░▐░█▄▄▄▄▄▄▄▄▄▐░█▄▄▄▄▄▄▄▄▄▐░▌ ▐░▌   ▐░▐░█▄▄▄▄▄▄▄▄▄▐░█▄▄▄▄▄▄▄█░▌
@@ -147,20 +149,21 @@ artText_Epic="""
 |  _____|   _ ( (_____  | |     |     __|  __)  |  __)  | (\ \) |  __)  |     __)
 | (     |  ( \ \      ) | |     | (\ (  | (     | (     | | \   | (     | (\ (   
 | )     |  /  \ /\____) | (____/| ) \ \_| (____/| (____/| )  \  | (____/| ) \ \__
-|/      |_/    \\_______(_______|/   \__(_______(_______|/    )_(_______|/   \__/
+|/      |_/    \ _______(_______|/   \__(_______(_______|/    )_(_______|/   \__/
+UPI: PKScreener@APL
 """
 artText_Isometric3="""
      @APL      ___         ___         ___         ___         ___         ___         ___         ___         ___  TM 🇮🇳
-    3/  /\    /__/|       /  /\       /  /\       /  /\       /  /\       /  /\       /__/\       /  /\       /  /\    
-   7/  /::\  |  |:|      /  /:/_     /  /:/      /  /::\     /  /:/_     /  /:/_      \  \:\     /  /:/_     /  /::\   
-  9/  /:/\:\ |  |:|     /  /:/ /\   /  /:/      /  /:/\:\   /  /:/ /\   /  /:/ /\      \  \:\   /  /:/ /\   /  /:/\:\  
- 2/  /:/~/:__|  |:|    /  /:/ /::\ /  /:/  ___ /  /:/~/:/  /  /:/ /:/_ /  /:/ /:/_ _____\__\:\ /  /:/ /:/_ /  /:/~/:/  
-6/__/:/ /:/__/\_|:|___/__/:/ /:/\:/__/:/  /  //__/:/ /:/__/__/:/ /:/ //__/:/ /:/ //__/::::::::/__/:/ /:/ //__/:/ /:/___
-1\  \:\/:/\  \:\/:::::\  \:\/:/~/:\  \:\ /  /:\  \:\/:::::\  \:\/:/ /:\  \:\/:/ /:\  \:\~~\~~\\  \:\/:/ /:\  \:\/::::::/ 
- 7\  \::/  \  \::/~~~~ \  \::/ /:/ \  \:\  /:/ \  \::/~~~~ \  \::/ /:/ \  \::/ /:/ \  \:\  ~~~ \  \::/ /:/ \  \::/~~~~/  
-  0\  \:\   \  \:\      \__\/ /:/   \  \:\/:/   \  \:\      \  \:\/:/   \  \:\/:/   \  \:\      \  \:\/:/   \  \:\     
-   0\  \:\   \  \:\       /__/:/     \  \::/     \  \:\      \  \::/     \  \::/     \  \:\      \  \::/     \  \:\    
-    8\__\/    \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/    
+    R/  /\    /__/|       /  /\       /  /\       /  /\       /  /\       /  /\       /__/\       /  /\       /  /\    
+   E/  /::\  |  |:|      /  /:/_     /  /:/      /  /::\     /  /:/_     /  /:/_      \  \:\     /  /:/_     /  /::\   
+  N/  /:/\:\ |  |:|     /  /:/ /\   /  /:/      /  /:/\:\   /  /:/ /\   /  /:/ /\      \  \:\   /  /:/ /\   /  /:/\:\  
+ E/  /:/~/:__|  |:|    /  /:/ /::\ /  /:/  ___ /  /:/~/:/  /  /:/ /:/_ /  /:/ /:/_ _____\__\:\ /  /:/ /:/_ /  /:/~/:/  
+E/__/:/ /:/__/\_|:|___/__/:/ /:/\:/__/:/  /  //__/:/ /:/__/__/:/ /:/ //__/:/ /:/ //__/::::::::/__/:/ /:/ //__/:/ /:/___
+R\  \:\/:/\  \:\/:::::\  \:\/:/~/:\  \:\ /  /:\  \:\/:::::\  \:\/:/ /:\  \:\/:/ /:\  \:\~~\~~ \  \:\/:/ /:\  \:\/::::::/ 
+ C\  \::/  \  \::/~~~~ \  \::/ /:/ \  \:\  /:/ \  \::/~~~~ \  \::/ /:/ \  \::/ /:/ \  \:\  ~~~ \  \::/ /:/ \  \::/~~~~/  
+  S\  \:\   \  \:\      \__\/ /:/   \  \:\/:/   \  \:\      \  \:\/:/   \  \:\/:/   \  \:\      \  \:\/:/   \  \:\     
+   K\  \:\   \  \:\       /__/:/     \  \::/     \  \:\      \  \::/     \  \::/     \  \:\      \  \::/     \  \:\    
+    P\__\/    \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/       \__\/    
 """
 artText_FlowerPower="""
 .-------..--.   .--.    .-'''-.    _______  .-------.       .-''-.     .-''-. ,---.   .--.   .-''-. .-------. TM 🇮🇳
@@ -172,19 +175,20 @@ artText_FlowerPower="""
 |   |    |  | \ `'   \    `-'  (  .  .-'_/  |  | \ `'   /\  `-'    /\  `-'    |  (_,_)\  |\  `-'    |  | \ `'   / 
 /   )    |  |  \    / \       / `-'`-'     /|  |  \    /  \       /  \       /|  |    |  | \       /|  |  \    /  
 `---'    `--'   `'-'   `-...-'    `._____.' ''-'   `'-'    `'-..-'    `'-..-' '--'    '--'  `'-..-' ''-'   `'-'   
+UPI: PKScreener@APL
 """
 artText_Impossible="""
          @APL       _           _            _            _          _           _           _            _           _   TM 🇮🇳
-        3/\ \      /\_\        / /\        /\ \          /\ \       /\ \        /\ \        /\ \     _   /\ \        /\ \    
-       7/  \ \    / / /  _    / /  \      /  \ \        /  \ \     /  \ \      /  \ \      /  \ \   /\_\/  \ \      /  \ \   
-      9/ /\ \ \  / / /  /\_\ / / /\ \__  / /\ \ \      / /\ \ \   / /\ \ \    / /\ \ \    / /\ \ \_/ / / /\ \ \    / /\ \ \  
-     2/ / /\ \_\/ / /__/ / // / /\ \___\/ / /\ \ \    / / /\ \_\ / / /\ \_\  / / /\ \_\  / / /\ \___/ / / /\ \_\  / / /\ \_\ 
-    6/ / /_/ / / /\_____/ / \ \ \ \/___/ / /  \ \_\  / / /_/ / // /_/_ \/_/ / /_/_ \/_/ / / /  \/____/ /_/_ \/_/ / / /_/ / / 
-   1/ / /__\/ / /\_______/   \ \ \    / / /    \/_/ / / /__\/ // /____/\   / /____/\   / / /    / / / /____/\   / / /__\/ /  
-  7/ / /_____/ / /\ \ \  _MADE\ \ \IN/ / /INDIA    / / /_____// /\____\/  / /\____\/  / / /    / / / /\____\/  / / /_____/   
- 0/ / /     / / /  \ \ \/_/\__/ / / / / /________ / / /\ \ \ / / /______ / / /______ / / /    / / / / /______ / / /\ \ \     
-0/ / /     / / /    \ \ \ \/___/ / / / /_________/ / /  \ \ / / /_______/ / /_______/ / /    / / / / /_______/ / /  \ \ \    
-8\/_/      \/_/      \_\_\_____\/  \/____________\/_/    \_\\/__________\/__________\/_/     \/_/\/__________\/_/    \_\/    
+        R/\ \      /\_\        / /\        /\ \          /\ \       /\ \        /\ \        /\ \     _   /\ \        /\ \    
+       E/  \ \    / / /  _    / /  \      /  \ \        /  \ \     /  \ \      /  \ \      /  \ \   /\_\/  \ \      /  \ \   
+      N/ /\ \ \  / / /  /\_\ / / /\ \__  / /\ \ \      / /\ \ \   / /\ \ \    / /\ \ \    / /\ \ \_/ / / /\ \ \    / /\ \ \  
+     E/ / /\ \_\/ / /__/ / // / /\ \___\/ / /\ \ \    / / /\ \_\ / / /\ \_\  / / /\ \_\  / / /\ \___/ / / /\ \_\  / / /\ \_\ 
+    E/ / /_/ / / /\_____/ / \ \ \ \/___/ / /  \ \_\  / / /_/ / // /_/_ \/_/ / /_/_ \/_/ / / /  \/____/ /_/_ \/_/ / / /_/ / / 
+   R/ / /__\/ / /\_______/   \ \ \    / / /    \/_/ / / /__\/ // /____/\   / /____/\   / / /    / / / /____/\   / / /__\/ /  
+  C/ / /_____/ / /\ \ \  _MADE\ \ \IN/ / /INDIA    / / /_____// /\____\/  / /\____\/  / / /    / / / /\____\/  / / /_____/   
+ S/ / /     / / /  \ \ \/_/\__/ / / / / /________ / / /\ \ \ / / /______ / / /______ / / /    / / / / /______ / / /\ \ \     
+K/ / /     / / /    \ \ \ \/___/ / / / /_________/ / /  \ \ / / /_______/ / /_______/ / /    / / / / /_______/ / /  \ \ \    
+P\/_/      \/_/      \_\_\_____\/  \/____________\/_/    \_\ /__________\/__________\/_/     \/_/\/__________\/_/    \_\/    
 """
 import random
 
@@ -194,4 +198,7 @@ def getArtText():
     artTexts = [artText_ansiRegular,artText_Merlin1,artText_dos_rebel,artText_Puffy,artText_Rounded,artText_Standard,artText_Varsity,artText_Collosol,artText_Roman,artText_Electronic,artText_Epic,artText_Isometric3,artText_FlowerPower,artText_Impossible]
     artTexts.extend(artTexts)
     random.shuffle(artTexts)
-    return random.choice(artTexts)
+    from PKDevTools.classes.System import PKSystem
+    from pkscreener.classes import VERSION
+    sysName,_,_,_,_ = PKSystem.get_platform()
+    return f"{random.choice(artTexts)}{sysName} | v{VERSION}"
