@@ -33,6 +33,8 @@ import pandas as pd
 import pkscreener.classes.Utility as Utility
 from pkscreener.classes.ConfigManager import parser, tools
 from pkscreener.classes.ScreeningStatistics import ScreeningStatistics
+from pkscreener.classes import AssetsManager
+
 from PKDevTools.classes.ColorText import colorText
 from PKDevTools.classes import Archiver
 from PKDevTools.classes.Singleton import SingletonType, SingletonMixin
@@ -44,6 +46,8 @@ from PKDevTools.classes.OutputControls import OutputControls
 from PKDevTools.classes.SuppressOutput import SuppressOutput
 from PKDevTools.classes.MarketHours import MarketHours
 
+from halo import Halo
+
 configManager = tools()
 
 STD_ENCODING=sys.stdout.encoding if sys.stdout is not None else 'utf-8'
@@ -77,20 +81,26 @@ class PKMarketOpenCloseAnalyser:
     updatedCandleData = None
     allDailyCandles = None
     allIntradayCandles = None
-    def getStockDataForSimulation():
-        int_exists, int_cache_file = PKMarketOpenCloseAnalyser.ensureIntradayStockDataExists()
-        daily_exists, daily_cache_file = PKMarketOpenCloseAnalyser.ensureDailyStockDataExists()
+    
+    def getStockDataForSimulation(sliceWindowDatetime=None,listStockCodes=[]):
+        int_exists, int_cache_file, stockDictInt = PKMarketOpenCloseAnalyser.ensureIntradayStockDataExists(listStockCodes=listStockCodes)
+        daily_exists, daily_cache_file, stockDict = PKMarketOpenCloseAnalyser.ensureDailyStockDataExists(listStockCodes=listStockCodes)
         updatedCandleData = PKMarketOpenCloseAnalyser.updatedCandleData
         allDailyCandles = PKMarketOpenCloseAnalyser.allDailyCandles
-        if  (int_exists and daily_exists) and (updatedCandleData is None or allDailyCandles is None):
-            allDailyCandles = PKMarketOpenCloseAnalyser.getLatestDailyCandleData(daily_cache_file)
-            morningIntradayCandle = PKMarketOpenCloseAnalyser.getIntradayCandleFromMorning(int_cache_file)
+        if updatedCandleData is not None and len(updatedCandleData) < 1:
+            updatedCandleData = None
+        if allDailyCandles is not None and len(allDailyCandles) < 1:
+            allDailyCandles = None
+        if  ((int_exists or len(stockDictInt) > 0) and (daily_exists or len(stockDict) > 0)) and (updatedCandleData is None or allDailyCandles is None):
+            allDailyCandles = PKMarketOpenCloseAnalyser.getLatestDailyCandleData(daily_cache_file,stockDict)
+            morningIntradayCandle = PKMarketOpenCloseAnalyser.getIntradayCandleFromMorning(int_cache_file,sliceWindowDatetime=sliceWindowDatetime,stockDictInt=stockDictInt)
             updatedCandleData = PKMarketOpenCloseAnalyser.combineDailyStockDataWithMorningSimulation(allDailyCandles,morningIntradayCandle)
-            # PKMarketOpenCloseAnalyser.updatedCandleData = updatedCandleData
-            # PKMarketOpenCloseAnalyser.allDailyCandles = allDailyCandles
-            Utility.tools.saveStockData(updatedCandleData,PKMarketOpenCloseAnalyser.configManager,1,False,False, True)
+            PKMarketOpenCloseAnalyser.updatedCandleData = updatedCandleData
+            PKMarketOpenCloseAnalyser.allDailyCandles = allDailyCandles
+            AssetsManager.PKAssetsManager.saveStockData(updatedCandleData,PKMarketOpenCloseAnalyser.configManager,1,False,False, True)
         return updatedCandleData, allDailyCandles
 
+    @Halo(text='  [+] Running final analysis...', spinner='dots')
     def runOpenCloseAnalysis(updatedCandleData,allDailyCandles,screen_df,save_df,runOptionName=None,filteredListOfStocks=[]):
         # stockListFromMorningTrade,morningIntraday_df = PKMarketOpenCloseAnalyser.simulateMorningTrade(updatedCandleData)
         # latest_daily_df = PKMarketOpenCloseAnalyser.runScanForStocksFromMorningTrade(stockListFromMorningTrade,allDailyCandles)
@@ -98,90 +108,112 @@ class PKMarketOpenCloseAnalyser:
             shouldSuppress = not OutputControls().enableMultipleLineOutput
             with SuppressOutput(suppress_stderr=shouldSuppress, suppress_stdout=shouldSuppress):
                 save_df, screen_df = PKMarketOpenCloseAnalyser.diffMorningCandleDataWithLatestDailyCandleData(screen_df,save_df, updatedCandleData, allDailyCandles,runOptionName=runOptionName,filteredListOfStocks=filteredListOfStocks)
-        except:
+        except: # pragma: no cover
             pass
-        Utility.tools.saveStockData(allDailyCandles,PKMarketOpenCloseAnalyser.configManager,1,False,False, True)
+        AssetsManager.PKAssetsManager.saveStockData(allDailyCandles,PKMarketOpenCloseAnalyser.configManager,1,False,False, True)
         return save_df, screen_df
 
-    def ensureIntradayStockDataExists():
+    @Halo(text='  [+] Getting intraday data...', spinner='dots')
+    def ensureIntradayStockDataExists(listStockCodes=[]):
         # Ensure that the intraday_stock_data_<date>.pkl file exists
-        exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=True)
-        copyFilePath = os.path.join(Archiver.get_user_outputs_dir(), f"copy_{cache_file}")
-        srcFilePath = os.path.join(Archiver.get_user_outputs_dir(), cache_file)
+        exists, cache_file = AssetsManager.PKAssetsManager.afterMarketStockDataExists(intraday=True)
+        copyFilePath = os.path.join(Archiver.get_user_data_dir(), f"copy_{cache_file}")
+        srcFilePath = os.path.join(Archiver.get_user_data_dir(), cache_file)
         srcFileSize = os.stat(srcFilePath).st_size if os.path.exists(srcFilePath) else 0
+        stockDict = None
         if exists and srcFileSize < 1024*1024*40:
              # File less than 30MB ? Must have been corrupted
             try:
                 os.remove(srcFilePath)
                 exists = False
-            except:
+            except: # pragma: no cover
                 pass
-        if not exists:
+        isTrading = PKDateUtilities.isTradingTime()
+        if not exists or isTrading:
             savedPeriod = PKMarketOpenCloseAnalyser.configManager.period
             savedDuration = PKMarketOpenCloseAnalyser.configManager.duration
             PKMarketOpenCloseAnalyser.configManager.period = "1d"
             PKMarketOpenCloseAnalyser.configManager.duration = "1m"
-            OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()} !")
-            OutputControls().printOutput(f"[+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
-            if os.path.exists(copyFilePath):
+            PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+            OutputControls().printOutput(f"  [+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_data_dir()} !")
+            OutputControls().printOutput(f"  [+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
+            if os.path.exists(copyFilePath) and not isTrading:
                 copyFileSize = os.stat(copyFilePath).st_size if os.path.exists(copyFilePath) else 0
                 if copyFileSize >= 1024*1024*40:
                     shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
-                    return True, cache_file
-            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,[],isIntraday=True)
-            exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=True)
-            if not exists:
-                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()}/ !")
-                OutputControls().printOutput(f"[+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d -i 1m{colorText.END} and then run this menu option again.")
-                input("Press any key to continue...")
+                    PKMarketOpenCloseAnalyser.configManager.period = savedPeriod
+                    PKMarketOpenCloseAnalyser.configManager.duration = savedDuration
+                    PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+                    return True, cache_file, stockDict
+            stockDict = AssetsManager.PKAssetsManager.loadStockData(stockDict={},configManager=PKMarketOpenCloseAnalyser.configManager,downloadOnly=False,defaultAnswer='Y',retrial=False,forceLoad=False,stockCodes=listStockCodes,isIntraday=True)
+            exists, cache_file = AssetsManager.PKAssetsManager.afterMarketStockDataExists(intraday=True)
             PKMarketOpenCloseAnalyser.configManager.period = savedPeriod
             PKMarketOpenCloseAnalyser.configManager.duration = savedDuration
+            PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+            if not exists and len(stockDict) <= 0:
+                OutputControls().printOutput(f"  [+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_data_dir()}/ !")
+                OutputControls().printOutput(f"  [+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d -i 1m{colorText.END} and then run this menu option again.")
+                OutputControls().takeUserInput("Press any key to continue...")
         try:
             if os.path.exists(copyFilePath) and exists:
                 shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
             if not os.path.exists(copyFilePath) and exists: # Let's make a copy of the original one
                 shutil.copy(srcFilePath,copyFilePath)
-        except:
+        except: # pragma: no cover
             pass
-        return exists, cache_file
+        return exists, cache_file, stockDict
 
-    def ensureDailyStockDataExists():
+    @Halo(text='  [+] Getting daily data...', spinner='dots')
+    def ensureDailyStockDataExists(listStockCodes=[]):
         # Ensure that the stock_data_<date>.pkl file exists
-        exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=False)
-        copyFilePath = os.path.join(Archiver.get_user_outputs_dir(), f"copy_{cache_file}")
-        srcFilePath = os.path.join(Archiver.get_user_outputs_dir(), cache_file)
+        exists, cache_file = AssetsManager.PKAssetsManager.afterMarketStockDataExists(intraday=False)
+        copyFilePath = os.path.join(Archiver.get_user_data_dir(), f"copy_{cache_file}")
+        srcFilePath = os.path.join(Archiver.get_user_data_dir(), cache_file)
         srcFileSize = os.stat(srcFilePath).st_size if os.path.exists(srcFilePath) else 0
+        stockDict = None
         if exists and srcFileSize < 1024*1024*40:
              # File less than 30MB ? Must have been corrupted
             try:
                 os.remove(srcFilePath)
                 exists = False
-            except:
+            except: # pragma: no cover
                 pass
-        if not exists:
-            OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()} !")
+        isTrading = PKDateUtilities.isTradingTime()
+        if not exists or isTrading:
+            savedPeriod = PKMarketOpenCloseAnalyser.configManager.period
+            savedDuration = PKMarketOpenCloseAnalyser.configManager.duration
+            PKMarketOpenCloseAnalyser.configManager.period = "1y"
+            PKMarketOpenCloseAnalyser.configManager.duration = "1d"
+            PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+            OutputControls().printOutput(f"  [+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_data_dir()} !")
         # We should download a fresh copy anyways because we may have altered the existing copy in
         # the previous run. -- !!!! Not required if we saved at the end of last operation !!!!
-            OutputControls().printOutput(f"[+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
-            if os.path.exists(copyFilePath):
+            OutputControls().printOutput(f"  [+] {colorText.GREEN}Trying to download {cache_file}{colorText.END}. Please wait ...")
+            if os.path.exists(copyFilePath) and not isTrading:
                 copyFileSize = os.stat(copyFilePath).st_size if os.path.exists(copyFilePath) else 0
                 if copyFileSize >= 1024*1024*40:
                     shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
-                    return True, cache_file
-            Utility.tools.loadStockData({},PKMarketOpenCloseAnalyser.configManager,False,'Y',False,False,[],isIntraday=False,forceRedownload=True)
-            exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=False)
-            if not exists:
-                OutputControls().printOutput(f"[+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_outputs_dir()}/ !")
-                OutputControls().printOutput(f"[+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d{colorText.END} and then run this menu option again.")
-                input("Press any key to continue...")
+                    PKMarketOpenCloseAnalyser.configManager.period = savedPeriod
+                    PKMarketOpenCloseAnalyser.configManager.duration = savedDuration
+                    PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+                    return True, cache_file, stockDict
+            stockDict = AssetsManager.PKAssetsManager.loadStockData(stockDict={},configManager=PKMarketOpenCloseAnalyser.configManager,downloadOnly=False,defaultAnswer='Y',retrial=False,forceLoad=False,stockCodes=listStockCodes,isIntraday=False,forceRedownload=True)
+            exists, cache_file = AssetsManager.PKAssetsManager.afterMarketStockDataExists(intraday=False)
+            PKMarketOpenCloseAnalyser.configManager.period = savedPeriod
+            PKMarketOpenCloseAnalyser.configManager.duration = savedDuration
+            PKMarketOpenCloseAnalyser.configManager.setConfig(parser, default=True, showFileCreatedText=False)
+            if not exists and len(stockDict) <= 0:
+                OutputControls().printOutput(f"  [+] {colorText.FAIL}{cache_file}{colorText.END} not found under {Archiver.get_user_data_dir()}/ !")
+                OutputControls().printOutput(f"  [+] Please run {colorText.FAIL}pkscreener{colorText.END}{colorText.GREEN} -a Y -e -d{colorText.END} and then run this menu option again.")
+                OutputControls().takeUserInput("Press any key to continue...")
         try:
             if os.path.exists(copyFilePath) and exists:
                 shutil.copy(copyFilePath,srcFilePath) # copy is the saved source of truth
             if not os.path.exists(copyFilePath) and exists: # Let's make a copy of the original one
                 shutil.copy(srcFilePath,copyFilePath)
-        except:
+        except: # pragma: no cover
             pass
-        return exists, cache_file
+        return exists, cache_file, stockDict
     
     def simulateMorningTrade(updatedCandleData):
         # 1. For each stock, remove the latest daily data for today from stock_data_<date>.pkl
@@ -197,8 +229,10 @@ class PKMarketOpenCloseAnalyser:
         morningIntraday_df = None
         return stockListFromMorningTrade, morningIntraday_df
     
-    def getLatestDailyCandleData(daily_cache_file):
+    def getLatestDailyCandleData(daily_cache_file,stockDict=None):
         allDailyCandles = None
+        if stockDict is not None and len(stockDict) > 0:
+            return stockDict
         dailyDB = PKDailyStockDataDB(fileName=daily_cache_file)
         allDailyCandles = dailyDB.pickler.pickler.unpickle(fileName=dailyDB.pickler.fileName)
         # latestDailyCandle = {}
@@ -209,16 +243,20 @@ class PKMarketOpenCloseAnalyser:
         #                       columns=allDailyCandles[stock]["columns"],
         #                       index=[allDailyCandles[stock]["index"][-1]])
         #         latestDailyCandle[stock] = df.to_dict("split")
-        #     except:
+        #     except: # pragma: no cover
         #         continue
         return allDailyCandles
     
-    def getIntradayCandleFromMorning(int_cache_file=None,candle1MinuteNumberSinceMarketStarted=0):
+    @Halo(text='  [+] Simulating morning alert...', spinner='dots')
+    def getIntradayCandleFromMorning(int_cache_file=None,candle1MinuteNumberSinceMarketStarted=0,sliceWindowDatetime=None,stockDictInt=None):
         if candle1MinuteNumberSinceMarketStarted <= 0:
             candle1MinuteNumberSinceMarketStarted = PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber
         morningIntradayCandle = None
-        intradayDB = PKIntradayStockDataDB(fileName=int_cache_file)
-        allDailyIntradayCandles = intradayDB.pickler.pickler.unpickle(fileName=intradayDB.pickler.fileName)
+        if stockDictInt is not None and len(stockDictInt) > 0:
+            allDailyIntradayCandles = stockDictInt
+        else:
+            intradayDB = PKIntradayStockDataDB(fileName=int_cache_file)
+            allDailyIntradayCandles = intradayDB.pickler.pickler.unpickle(fileName=intradayDB.pickler.fileName)
         PKMarketOpenCloseAnalyser.allIntradayCandles = allDailyIntradayCandles
         morningIntradayCandle = {}
         stocks = list(allDailyIntradayCandles.keys())
@@ -236,41 +274,63 @@ class PKMarketOpenCloseAnalyser:
                 df = pd.DataFrame(data=allDailyIntradayCandles[stock]["data"],
                                 columns=allDailyIntradayCandles[stock]["columns"],
                                 index=allDailyIntradayCandles[stock]["index"])
-                df = df.head(numOfCandles)
+                if sliceWindowDatetime is None:
+                    df = df.head(numOfCandles)
                 try:
-                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30').to_datetime64()]
-                except:
-                    df = df[df.index <=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30', utc=True)]
+                    alertCandleTimestamp = sliceWindowDatetime if sliceWindowDatetime is not None else f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30'
+                    df = df[df.index <=  pd.to_datetime(alertCandleTimestamp).to_datetime64()]
+                except: # pragma: no cover
+                    alertCandleTimestamp = sliceWindowDatetime if sliceWindowDatetime is not None else f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+candle1MinuteNumberSinceMarketStarted}:00+05:30'
+                    df = df[df.index <=  pd.to_datetime(alertCandleTimestamp, utc=True)]
                     pass
+                with pd.option_context('mode.chained_assignment', None):
+                    df.dropna(axis=0, how="all", inplace=True)
                 if df is not None and len(df) > 0:
-                    combinedCandle = {"Open":PKMarketOpenCloseAnalyser.getMorningOpen(df), "High":max(df["High"]), 
-                                    "Low":min(df["Low"]),"Close":PKMarketOpenCloseAnalyser.getMorningClose(df),
-                                    "Adj Close":df["Adj Close"][-1],"Volume":sum(df["Volume"])}
+                    close = PKMarketOpenCloseAnalyser.getMorningClose(df)
+                    adjClose = df["Adj Close"][-1] if "Adj Close" in df.columns else close
+                    combinedCandle = {"open":PKMarketOpenCloseAnalyser.getMorningOpen(df), "high":max(df["high"]), 
+                                    "low":min(df["low"]),"close":close,
+                                    "Adj Close":adjClose,"volume":sum(df["volume"])}
                     tradingDate = df.index[-1] #PKDateUtilities.tradingDate()
                     timestamp = datetime.datetime.strptime(tradingDate.strftime("%Y-%m-%d %H:%M:%S"),"%Y-%m-%d %H:%M:%S")
                     df = pd.DataFrame([combinedCandle], columns=df.columns, index=[timestamp])
                     morningIntradayCandle[stock] = df.to_dict("split")
-            except Exception as e:
+            except KeyboardInterrupt: # pragma: no cover
+                raise KeyboardInterrupt
+            except Exception as e: # pragma: no cover
                 OutputControls().printOutput(f"{stock}:    {e}")
                 continue
         return morningIntradayCandle
 
     def getMorningOpen(df):
-        open = df["Open"][0]
+        try:
+            open = df["open"][0]
+        except KeyError: # pragma: no cover
+            open = df["open"][df.index.values[0]]
         index = 0
-        while open is np.nan and index < len(df):
-            open = df["Open"][index + 1]
+        while np.isnan(open) and index < len(df):
+            try:
+                open = df["open"][index + 1]
+            except KeyError: # pragma: no cover
+                open = df["open"][df.index.values[index + 1]]
             index += 1
         return open
     
     def getMorningClose(df):
-        close = df["Close"][-1]
+        try:
+            close = df["close"][-1]
+        except KeyError: # pragma: no cover
+            close = df["close"][df.index.values[-1]]
         index = len(df)
-        while close is np.nan and index >= 0:
-            close = df["Close"][index - 1]
+        while np.isnan(close) and index >= 0:
+            try:
+                close = df["close"][index - 1]
+            except KeyError: # pragma: no cover
+                close = df["close"][df.index.values[index - 1]]
             index -= 1
         return close
     
+    @Halo(text='  [+] Updating candles...', spinner='dots')
     def combineDailyStockDataWithMorningSimulation(allDailyCandles,morningIntradayCandle):
         mutableAllDailyCandles = copy.deepcopy(allDailyCandles)
         stocks = list(mutableAllDailyCandles.keys())
@@ -297,7 +357,7 @@ class PKMarketOpenCloseAnalyser:
                     # but for whatever reason, if we don't have the stock, we should skip those
                     # stocks from analysis
                     del mutableAllDailyCandles[stock]
-            except:
+            except: # pragma: no cover
                 del mutableAllDailyCandles[stock]
                 if 'PKDevTools_Default_Log_Level' in os.environ.keys():
                     intradayChange = colorText.miniTabulator().tabulate(
@@ -345,14 +405,23 @@ class PKMarketOpenCloseAnalyser:
         ts = None
         row = None
         scrStats = ScreeningStatistics(PKMarketOpenCloseAnalyser.configManager, default_logger())
+        tradingDate = PKDateUtilities.tradingDate()
+        DEFAULT_ALERT_TIME = PKDateUtilities.currentDateTime().replace(year=tradingDate.year,month=tradingDate.month,day=tradingDate.day,hour=MarketHours().openHour,minute=MarketHours().openMinute+configManager.morninganalysiscandlenumber)
+        morningAlertTime = DEFAULT_ALERT_TIME
         for stock in stocks:
             try:
                 # Open, High, Low, Close, Adj Close, Volume. We need the 3rd index item: Close.
                 dayHighLTP = allDailyCandles[stock]["data"][-1][1]
                 endOfDayLTP = allDailyCandles[stock]["data"][-1][3]
-                savedMorningLTP = updatedCandleData[stock]["data"][-1][3]
+                try:
+                    savedMorningLTP = updatedCandleData[stock]["data"][-1][3]
+                    morningTime = PKDateUtilities.utc_to_ist(updatedCandleData[stock]["index"][-1]).strftime("%H:%M")
+                    morningAlertTime = updatedCandleData[stock]["index"][-1]
+                except: # pragma: no cover
+                    savedMorningLTP = round(save_df["LTP"][index],2)
+                    morningTime = DEFAULT_ALERT_TIME.strftime("%H:%M")
+                    morningAlertTime = DEFAULT_ALERT_TIME
                 morningLTP = savedMorningLTP if pd.notna(savedMorningLTP) else round(save_df["LTP"][index],2)
-                morningTime = updatedCandleData[stock]["index"][-1].strftime("%H:%M")
                 morningTimestamps.append(morningTime)
                 morningCandles = PKMarketOpenCloseAnalyser.allIntradayCandles
                 df = pd.DataFrame(data=morningCandles[stock]["data"],
@@ -361,11 +430,11 @@ class PKMarketOpenCloseAnalyser:
                 # try:
                 #     # Let's only consider those candles that are after the alert issue-time in the mornings
                 #     df = df[df.index >=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} 09:{15+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber}:00+05:30').to_datetime64()]
-                # except:
+                # except: # pragma: no cover
                 #     df = df[df.index >=  pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} 09:{15+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber}:00+05:30', utc=True)]
                 #     pass
                 ts, row = scrStats.findMACDCrossover(df=df,
-                                           afterTimestamp=updatedCandleData[stock]["index"][-1],
+                                           afterTimestamp=morningAlertTime,
                                            nthCrossover=1,
                                            upDirection=True)
                 # saveDictionary = {}
@@ -378,7 +447,7 @@ class PKMarketOpenCloseAnalyser:
                 #         # Let's only consider those candles that are right after the alert issue-time in the mornings
                 #         index = pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber+nextSellMinute}:00+05:30').to_datetime64()
                 #         df = df[df.index <=  index]
-                #     except:
+                #     except: # pragma: no cover
                 #         index = pd.to_datetime(f'{PKDateUtilities.tradingDate().strftime(f"%Y-%m-%d")} {MarketHours().openHour:02}:{MarketHours().openMinute+PKMarketOpenCloseAnalyser.configManager.morninganalysiscandlenumber+nextSellMinute}:00+05:30', utc=True)
                 #         df = df[df.index <=  index]
                 #         pass
@@ -390,22 +459,34 @@ class PKMarketOpenCloseAnalyser:
                 highTS, highRow = scrStats.findIntradayHighCrossover(df=df)
                 # buySell_df = scrStats.computeBuySellSignals(updatedCandleData[stock]["data"])
                 # OutputControls().printOutput(buySell_df)
-                dayHighLTP = dayHighLTP if pd.notna(dayHighLTP) else highRow["High"][-1]
-                sellTimestamps.append(ts.strftime("%H:%M"))
-                dayHighTimestamps.append(highTS.strftime("%H:%M"))
-                sellLTPs.append(row["High"][-1])
+                dayHighLTP = dayHighLTP if pd.notna(dayHighLTP) else highRow["high"][-1]
+                sellTimestamps.append(PKDateUtilities.utc_to_ist(ts).strftime("%H:%M"))
+                dayHighTimestamps.append(PKDateUtilities.utc_to_ist(highTS).strftime("%H:%M"))
+                sellLTPs.append(row["high"][-1])
                 eodLTPs.append(round(endOfDayLTP,2))
                 dayHighLTPs.append(round(dayHighLTP,2))
                 eodDiffs.append(round(endOfDayLTP - morningLTP,2))
                 dayHighDiffs.append(round(dayHighLTP - morningLTP,2))
-                sqrOffDiffs.append(round(row["High"][-1] - morningLTP,2))
+                sqrOffDiffs.append(round(row["high"][-1] - morningLTP,2))
                 morningAlertLTPs.append(str(int(round(morningLTP,0))))
                 index += 1
-            except:
+            except: # pragma: no cover
                 eodLTPs.append("0")
                 eodDiffs.append("0")
                 dayHighLTPs.append("0")
                 dayHighDiffs.append("0")
+                if len(morningAlertLTPs) < len(eodLTPs):
+                    morningAlertLTPs.append("0")
+                if len(morningTimestamps) < len(eodLTPs):
+                    morningTimestamps.append("09:30")
+                if len(sellTimestamps) < len(eodLTPs):
+                    sellTimestamps.append("09:40")
+                if len(sellLTPs) < len(eodLTPs):
+                    sellLTPs.append("0")
+                if len(sqrOffDiffs) < len(eodLTPs):
+                    sqrOffDiffs.append("0")
+                if len(dayHighTimestamps) < len(eodLTPs):
+                    dayHighTimestamps.append("09:45")
                 continue
         diffColumns = ["LTP@Alert", "AlertTime", "SqrOff", "SqrOffLTP", "SqrOffDiff","DayHighTime","DayHigh","DayHighDiff", "EoDLTP", "EoDDiff"]
         diffValues = [morningAlertLTPs, morningTimestamps, sellTimestamps, sellLTPs, sqrOffDiffs,dayHighTimestamps,dayHighLTPs, dayHighDiffs,eodLTPs, eodDiffs]
@@ -413,11 +494,12 @@ class PKMarketOpenCloseAnalyser:
             columnName = column
             save_df[columnName] = diffValues[diffColumns.index(columnName)]
             screen_df.loc[:, columnName] = save_df.loc[:, columnName].apply(
-                lambda x: x if columnName in ["LTP@Alert","AlertTime", "SqrOff", "SqrOffLTP", "EoDLTP","DayHigh","DayHighTime"] else ((colorText.GREEN if x >= 0 else colorText.FAIL) + str(x) + colorText.END)
+                lambda x: x if columnName in ["LTP@Alert","AlertTime", "SqrOff", "SqrOffLTP", "EoDLTP","DayHigh","DayHighTime"] else ((colorText.GREEN if float(x) >= 0 else colorText.FAIL) + str(x) + colorText.END)
             )
 
         columns = save_df.columns
         lastIndex = len(save_df)
+        ltpSum = 0
         for col in columns:
             if col in ["Stock", "LTP@Alert", "Pattern", "LTP", "SqrOffLTP","SqrOffDiff","DayHigh","DayHighDiff", "EoDLTP", "EoDDiff", "%Chng"]:
                 if col == "Stock":
@@ -436,9 +518,9 @@ class PKMarketOpenCloseAnalyser:
         eodDiff = save_df.loc[lastIndex,"EoDDiff"]
         sqrOffDiff = save_df.loc[lastIndex,"SqrOffDiff"]
         dayHighDiff = save_df.loc[lastIndex,"DayHighDiff"]
-        save_df.loc[lastIndex,"EoDDiff"] = str(eodDiff) + f'({round(100*eodDiff/ltpSum,2)}%)'
-        save_df.loc[lastIndex,"SqrOffDiff"] = str(sqrOffDiff) + f'({round(100*sqrOffDiff/ltpSum,2)}%)'
-        save_df.loc[lastIndex,"DayHighDiff"] = str(dayHighDiff) + f'({round(100*dayHighDiff/ltpSum,2)}%)'
+        save_df.loc[lastIndex,"EoDDiff"] = str(eodDiff) + f'({round(100*eodDiff/ltpSum,2) if ltpSum >0 else 0}%)'
+        save_df.loc[lastIndex,"SqrOffDiff"] = str(sqrOffDiff) + f'({round(100*sqrOffDiff/ltpSum,2) if ltpSum >0 else 0}%)'
+        save_df.loc[lastIndex,"DayHighDiff"] = str(dayHighDiff) + f'({round(100*dayHighDiff/ltpSum,2) if ltpSum >0 else 0}%)'
         screen_df.loc[lastIndex,"EoDDiff"] = (colorText.GREEN if eodDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"EoDDiff"] + colorText.END
         screen_df.loc[lastIndex,"SqrOffDiff"] = (colorText.GREEN if sqrOffDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"SqrOffDiff"] + colorText.END
         screen_df.loc[lastIndex,"DayHighDiff"] = (colorText.GREEN if dayHighDiff >= 0 else colorText.FAIL) + save_df.loc[lastIndex,"DayHighDiff"] + colorText.END
@@ -448,7 +530,7 @@ class PKMarketOpenCloseAnalyser:
         screen_df.replace(np.nan, "", regex=True)
         save_df.replace(np.nan, "", regex=True)
         # Drop the unnecessary columns for this scanner type to make way for other columns to be fitted nicely on screen
-        columnsToBeDropped = ["Breakout(22Prds)","MA-Signal","Trend(22Prds)","index","EoDLTP"]
+        columnsToBeDropped = ["Breakout(22Prds)","index","EoDLTP","RS_Rating^NSEI","RVM(15)"]
         for col in columnsToBeDropped:
             if col in save_df.columns:
                 save_df.drop(col, axis=1, inplace=True, errors="ignore")
