@@ -28,6 +28,7 @@ import time
 import pandas as pd
 import multiprocessing
 from time import sleep
+from halo import Halo
 
 from PKDevTools.classes import Archiver
 from PKDevTools.classes.ColorText import colorText
@@ -44,9 +45,11 @@ from pkscreener.classes.CandlePatterns import CandlePatterns
 from pkscreener.classes.ConfigManager import parser, tools
 from PKDevTools.classes.OutputControls import OutputControls
 from PKNSETools.PKIntraDay import Intra_Day
+
 import pkscreener.classes.Fetcher as Fetcher
 import pkscreener.classes.ScreeningStatistics as ScreeningStatistics
 import pkscreener.classes.Utility as Utility
+from pkscreener.classes import AssetsManager
 
 class PKScanRunner:
     configManager = tools()
@@ -68,7 +71,7 @@ class PKScanRunner:
                 "52Wk-H",
                 "52Wk-L",
                 "%Chng",
-                "Volume",
+                "volume",
                 "MA-Signal",
                 "RSI",
                 "RSIi",
@@ -86,7 +89,7 @@ class PKScanRunner:
                 "52Wk-H",
                 "52Wk-L",
                 "%Chng",
-                "Volume",
+                "volume",
                 "MA-Signal",
                 "RSI",
                 "RSIi",
@@ -131,9 +134,44 @@ class PKScanRunner:
         actualHistoricalDuration = (samplingDuration - fillerPlaceHolder)
         return samplingDuration,fillerPlaceHolder,actualHistoricalDuration
 
-    def addStocksToItemList(userArgs, testing, testBuild, newlyListedOnly, downloadOnly, minRSI, maxRSI, insideBarToLookback, respChartPattern, daysForLowestVolume, backtestPeriod, reversalOption, maLength, listStockCodes, menuOption, exchangeName,executeOption, volumeRatio, items, daysInPast):
+    def addScansWithDefaultParams(userArgs, testing, testBuild, newlyListedOnly, downloadOnly, backtestPeriod, listStockCodes, menuOption, exchangeName,executeOption, volumeRatio, items, daysInPast,runOption=""):
+        import json
+        defaultOptionsDict = {}
+        filePath = os.path.join(Archiver.get_user_data_dir(),"defaults.json")
+        if not os.path.exists(filePath):
+            fileDownloaded = AssetsManager.PKAssetsManager.downloadSavedDefaultsFromServer("defaults.json")
+        if not os.path.exists(filePath):
+            return items
+        with open(filePath,"r") as f:
+            defaultOptionsDict = json.loads(f.read())
+        for scanOption in defaultOptionsDict.keys():
+            items = PKScanRunner.addStocksToItemList(userArgs=userArgs,
+                                                     testing=testing,
+                                                     testBuild=testBuild,
+                                                     newlyListedOnly=newlyListedOnly,
+                                                     downloadOnly=downloadOnly,
+                                                     minRSI=defaultOptionsDict[scanOption]["minRSI"],
+                                                     maxRSI=defaultOptionsDict[scanOption]["maxRSI"],
+                                                     insideBarToLookback=defaultOptionsDict[scanOption]["insideBarToLookback"],
+                                                     respChartPattern=defaultOptionsDict[scanOption]["respChartPattern"],
+                                                     daysForLowestVolume=defaultOptionsDict[scanOption]["daysForLowestVolume"],
+                                                     backtestPeriod=backtestPeriod,
+                                                     reversalOption=defaultOptionsDict[scanOption]["reversalOption"],
+                                                     maLength=defaultOptionsDict[scanOption]["maLength"],
+                                                     listStockCodes=listStockCodes,
+                                                     menuOption=menuOption,
+                                                     exchangeName=exchangeName,
+                                                     executeOption=int(scanOption.split(":")[2]), 
+                                                     volumeRatio=volumeRatio,
+                                                     items=items,
+                                                     daysInPast=daysInPast,
+                                                     runOption=scanOption)
+        return items
+    
+    def addStocksToItemList(userArgs, testing, testBuild, newlyListedOnly, downloadOnly, minRSI, maxRSI, insideBarToLookback, respChartPattern, daysForLowestVolume, backtestPeriod, reversalOption, maLength, listStockCodes, menuOption, exchangeName,executeOption, volumeRatio, items, daysInPast,runOption=""):
         moreItems = [
                         (
+                            runOption,
                             menuOption,
                             exchangeName,
                             executeOption,
@@ -159,7 +197,7 @@ class PKScanRunner:
                                 else PKScanRunner.configManager.effectiveDaysToLookback
                             ),
                             default_logger().level,
-                            (menuOption in ["B", "G", "X", "S","C"])
+                            (menuOption in ["B", "G", "X", "S","C", "F"])
                             or (userArgs.backtestdaysago is not None),
                             # assumption is that fetcher.fetchStockData would be
                             # mocked to avoid calling yf.download again and again
@@ -168,6 +206,7 @@ class PKScanRunner:
                         for stock in listStockCodes
                     ]
         items.extend(moreItems)
+        return items
 
     def getStocksListForScan(userArgs, menuOption, totalStocksInReview, downloadedRecently, daysInPast):
         savedStocksCount = 0
@@ -243,23 +282,31 @@ class PKScanRunner:
         if choices.endswith("_"):
             choices = choices[:-1]
         choices = f"{choices}{'_i' if isIntraday else ''}"
-        return choices
+        return f'{choices.strip()}{"_IA" if userArgs is not None and userArgs.runintradayanalysis else ""}'
 
     def refreshDatabase(consumers,stockDictPrimary,stockDictSecondary):
         for worker in consumers:
             worker.objectDictionaryPrimary = stockDictPrimary
             worker.objectDictionarySecondary = stockDictSecondary
             worker.refreshDatabase = True
-            
+    
+    # @Halo(text='', spinner='dots')
     def runScanWithParams(userPassedArgs,keyboardInterruptEvent,screenCounter,screenResultsCounter,stockDictPrimary,stockDictSecondary,testing, backtestPeriod, menuOption, executeOption, samplingDuration, items,screenResults, saveResults, backtest_df,scanningCb,tasks_queue, results_queue, consumers,logging_queue):
         if tasks_queue is None or results_queue is None or consumers is None:
-            tasks_queue, results_queue, consumers,logging_queue = PKScanRunner.prepareToRunScan(menuOption,keyboardInterruptEvent,screenCounter, screenResultsCounter, stockDictPrimary,stockDictSecondary, items,executeOption,userPassedArgs)
+            try:
+                import tensorflow as tf
+                with tf.device("/device:GPU:0"):
+                    tasks_queue, results_queue, consumers,logging_queue = PKScanRunner.prepareToRunScan(menuOption,keyboardInterruptEvent,screenCounter, screenResultsCounter, stockDictPrimary,stockDictSecondary, items,executeOption,userPassedArgs)
+            except: # pragma: no cover
+                tasks_queue, results_queue, consumers,logging_queue = PKScanRunner.prepareToRunScan(menuOption,keyboardInterruptEvent,screenCounter, screenResultsCounter, stockDictPrimary,stockDictSecondary, items,executeOption,userPassedArgs)
+                pass
             try:
                 if logging_queue is not None:
                     log_queue_reader = LogQueueReader(logging_queue)
                     log_queue_reader.start()
-            except:
+            except: # pragma: no cover
                 pass
+
         # if executeOption == 29: # Intraday Bid/Ask, for which we need to fetch data from NSE instead of yahoo
         #     intradayFetcher = Intra_Day("SBINEQN") # This will initialise the cookies etc.
         #     for consumer in consumers:
@@ -286,7 +333,7 @@ class PKScanRunner:
                 )
 
         OutputControls().printOutput(colorText.END)
-        if userPassedArgs is not None and (userPassedArgs.monitor is None and "|" not in userPassedArgs.options) and not userPassedArgs.options.upper().startswith("C"):
+        if userPassedArgs is not None and not userPassedArgs.testalloptions and (userPassedArgs.monitor is None and "|" not in userPassedArgs.options) and not userPassedArgs.options.upper().startswith("C"):
             # Don't terminate the multiprocessing clients if we're 
             # going to pipe the results from an earlier run
             # or we're running in monitoring mode
@@ -298,11 +345,19 @@ class PKScanRunner:
         return screenResults, saveResults,backtest_df,tasks_queue, results_queue, consumers, logging_queue
 
     @exit_after(180) # Should not remain stuck starting the multiprocessing clients beyond this time
+    @Halo(text='  [+] Creating multiple processes for faster processing...', spinner='dots')
     def prepareToRunScan(menuOption,keyboardInterruptEvent, screenCounter, screenResultsCounter, stockDictPrimary,stockDictSecondary, items, executeOption,userPassedArgs):
         tasks_queue, results_queue, totalConsumers, logging_queue = PKScanRunner.initQueues(len(items),userPassedArgs)
         scr = ScreeningStatistics.ScreeningStatistics(PKScanRunner.configManager, default_logger())
-        exists, cache_file = Utility.tools.afterMarketStockDataExists(intraday=PKScanRunner.configManager.isIntradayConfig())
+        exists, cache_file = AssetsManager.PKAssetsManager.afterMarketStockDataExists(intraday=PKScanRunner.configManager.isIntradayConfig())
         sec_cache_file = cache_file if "intraday_" in cache_file else f"intraday_{cache_file}"
+        # Get RS rating stock value of the index
+        from pkscreener.classes.Fetcher import screenerStockDataFetcher
+        nsei_df = screenerStockDataFetcher().fetchStockData(PKScanRunner.configManager.baseIndex,PKScanRunner.configManager.period,PKScanRunner.configManager.duration,None,0,0,0,exchangeSuffix="",printCounter=False)
+        rs_score_index = -1
+        PKScanRunner.configManager.getConfig(parser)
+        if nsei_df is not None:
+            rs_score_index = scr.calc_relative_strength(nsei_df[::-1])
         consumers = [
                     PKMultiProcessorClient(
                         StockScreener().screenStocks,
@@ -326,6 +381,7 @@ class PKScanRunner:
                         # None
                         (cache_file if (exists and menuOption in ["C"]) else None),
                         (sec_cache_file if (exists and menuOption in ["C"]) else None),
+                        rs_strange_index=rs_score_index
                     )
                     for _ in range(totalConsumers)
                 ]
@@ -333,7 +389,7 @@ class PKScanRunner:
         try:
             intradayFetcher = None
             intradayFetcher = Intra_Day("SBINEQN") # This will initialise the cookies etc.
-        except:
+        except: # pragma: no cover
             pass
         for consumer in consumers:
             consumer.intradayNSEFetcher = intradayFetcher
@@ -341,6 +397,7 @@ class PKScanRunner:
         return tasks_queue,results_queue,consumers,logging_queue
 
     @exit_after(120) # Should not remain stuck starting the multiprocessing clients beyond this time
+    @Halo(text='', spinner='dots')
     def startWorkers(consumers):
         try:
             from pytest_cov.embed import cleanup_on_signal, cleanup_on_sigterm
@@ -354,9 +411,8 @@ class PKScanRunner:
             else:
                 cleanup_on_sigterm()
         OutputControls().printOutput(
-            colorText.BOLD
-            + colorText.FAIL
-            + f"[+] Using Period:{PKScanRunner.configManager.period} and Duration:{PKScanRunner.configManager.duration} for scan! You can change this in user config."
+            colorText.FAIL
+            + f"\n  [+] Using Period:{colorText.END}{colorText.GREEN}{PKScanRunner.configManager.period}{colorText.END}{colorText.FAIL} and Duration:{colorText.END}{colorText.GREEN}{PKScanRunner.configManager.duration}{colorText.END}{colorText.FAIL} for scan! You can change this in user config."
             + colorText.END
         )
         start_time = time.time()
@@ -366,8 +422,10 @@ class PKScanRunner:
             worker.start()
         OutputControls().printOutput(f"Started all workers in {round(time.time() - start_time,4)}s")
         if OutputControls().enableMultipleLineOutput:
-            sys.stdout.write("\x1b[1A") # Move cursor up to hide the starting times we printed above
+            # sys.stdout.write("\x1b[1A") # Move cursor up to hide the starting times we printed above
+            OutputControls().moveCursorUpLines(1)
 
+    @Halo(text='', spinner='dots')
     def terminateAllWorkers(userPassedArgs,consumers, tasks_queue, testing=False):
         shouldSuppress = (userPassedArgs is None) or (userPassedArgs is not None and not userPassedArgs.log)
         with SuppressOutput(suppress_stderr=shouldSuppress, suppress_stdout=shouldSuppress):
@@ -386,7 +444,7 @@ class PKScanRunner:
                         # while worker.is_alive():
                     worker.terminate()
                     default_logger().debug("Worker terminated!")
-                    # except:
+                    # except: # pragma: no cover
                     #     continue
                 except OSError as e: # pragma: no cover
                     default_logger().debug(e, exc_info=True)
@@ -397,6 +455,8 @@ class PKScanRunner:
             while True:
                 try:
                     _ = tasks_queue.get(False)
+                except KeyboardInterrupt: # pragma: no cover
+                    raise KeyboardInterrupt
                 except Exception as e:  # pragma: no cover
                     # default_logger().debug(e, exc_info=True)
                     break
@@ -408,7 +468,7 @@ class PKScanRunner:
     def shutdown(frame, signum):
         OutputControls().printOutput("Shutting down for test coverage")
 
-    # @exit_after(60)
+    # @Halo(text='', spinner='dots')
     def runScan(userPassedArgs,testing,numStocks,iterations,items,numStocksPerIteration,tasks_queue,results_queue,originalNumberOfStocks,backtest_df, *otherArgs,resultsReceivedCb=None):
         queueCounter = 0
         counter = 0
