@@ -38,37 +38,37 @@ from pkscreener import Imports
 if Imports["talib"]:
     try:
         import talib
-    except:
+    except Exception as e: # pragma: no cover
+        issueLink = "https://github.com/pkjmesra/PKScreener"
+        issueLink = f"\x1b[97m\x1b]8;;{issueLink}\x1b\\{issueLink}\x1b]8;;\x1b\\\x1b[0m"
+        taLink = "https://github.com/ta-lib/ta-lib-python"
+        taLink = f"\x1b[97m\x1b]8;;{taLink}\x1b\\{taLink}\x1b]8;;\x1b\\\x1b[0m"
         OutputControls().printOutput(
-                colorText.BOLD
-                + colorText.FAIL
-                + "[+] 'TA-Lib' library is not installed. For best results, please install 'TA-Lib'! You may wish to follow instructions from\n[+] https://github.com/pkjmesra/PKScreener/"
+                colorText.FAIL
+                + f"  [+] 'TA-Lib' library is not installed. For best results, please install 'TA-Lib'! You may wish to follow instructions from\n  [+] {issueLink}"
                 + colorText.END
             )
         try:
-            import pandas_ta as talib
+            import pandas_ta_classic as talib
             OutputControls().printOutput(
-                colorText.BOLD
-                + colorText.FAIL
-                + "[+] TA-Lib is not installed. Falling back on pandas_ta.\n[+] For full coverage(candle patterns), you may wish to follow instructions from\n[+] https://github.com/ta-lib/ta-lib-python"
+                colorText.FAIL
+                + f"  [+] TA-Lib is not installed. Falling back on pandas_ta_classic.\n  [+] For full coverage(candle patterns), you may wish to follow instructions from\n  [+] {taLink}"
                 + colorText.END
             )
-        except:
+        except: # pragma: no cover
             OutputControls().printOutput(
-                colorText.BOLD
-                + colorText.FAIL
-                + "[+] pandas_ta is not installed. Falling back on pandas_ta also failed.\n[+] For full coverage(candle patterns), you may wish to follow instructions from\n[+] https://github.com/ta-lib/ta-lib-python"
+                colorText.FAIL
+                + f"  [+] pandas_ta_classic is not installed. Falling back on pandas_ta_classic also failed.\n  [+] For full coverage(candle patterns), you may wish to follow instructions from\n  [+] {taLink}"
                 + colorText.END
             )
             pass
         pass
 else:
     try:
-        import pandas_ta as talib
+        import pandas_ta_classic as talib
         OutputControls().printOutput(
-            colorText.BOLD
-            + colorText.FAIL
-            + "[+] TA-Lib is not installed. Falling back on pandas_ta.\n[+] For full coverage(candle patterns), you may wish to follow instructions from\n[+] https://github.com/ta-lib/ta-lib-python"
+            colorText.FAIL
+            + "  [+] TA-Lib is not installed. Falling back on pandas_ta_classic.\n  [+] For full coverage(candle patterns), you may wish to follow instructions from\n  [+] https://github.com/ta-lib/ta-lib-python"
             + colorText.END
         )
         sleep(3)
@@ -78,6 +78,18 @@ else:
 
 
 class pktalib:
+
+    @classmethod
+    def align_series(*series_list, fill_value=0):
+        """
+        Aligns multiple Pandas Series to have the same index and returns them.
+        Missing values are filled with the given fill_value.
+        """
+        if all(isinstance(s, pd.Series) for s in series_list):
+            aligned_series = pd.concat(series_list, axis=1).fillna(fill_value)
+            return [aligned_series[col] for col in aligned_series.columns]
+        return series_list
+
     @classmethod
     def AVWAP(self,df,anchored_date:pd.Timestamp):
         # anchored_date = pd.to_datetime('2022-01-30')
@@ -91,12 +103,12 @@ class pktalib:
         # identify potential areas of support and resistance more accurately 
         # and make better trading decisions.
         with pd.option_context('mode.chained_assignment', None):
-            df["VWAP_D"] = pktalib.VWAP(high=df["High"],low=df["Low"],close=df["Close"],volume=df["Volume"],anchor="D")
+            df["VWAP_D"] = pktalib.VWAP(high=df["high"],low=df["low"],close=df["close"],volume=df["volume"],anchor="D")
             # If we create a column 'typical_price', it should be identical with 'VWAP_D'
-            df['typical_price'] = (df['High'] + df['Low'] + df['Close'])/3
-            tpp_d = ((df['High'] + df['Low'] + df['Close'])*df['Volume'])/3
+            df['typical_price'] = (df["high"] + df["low"] + df["close"])/3
+            tpp_d = ((df["high"] + df["low"] + df["close"])*df["volume"])/3
             df['anchored_VWAP'] = tpp_d.where(df.index >= anchored_date).groupby(
-                df.index >= anchored_date).cumsum()/df['Volume'].where(
+                df.index >= anchored_date).cumsum()/df["volume"].where(
                     df.index >= anchored_date).groupby(
                         df.index >= anchored_date).cumsum()
         return df['anchored_VWAP']
@@ -120,7 +132,9 @@ class pktalib:
     @classmethod
     def VWAP(self, high, low, close, volume,anchor=None):
         try:
-            import pandas_ta as talib
+            import pandas_ta_classic as talib
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
             return talib.vwap(high, low, close, volume,anchor=anchor)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -131,6 +145,8 @@ class pktalib:
         try:
             low_kel = None
             upp_kel = None
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
             tr = pktalib.TRUERANGE(high, low, close)
             atr = pktalib.ATR(high, low, close, timeperiod=timeperiod)
             sma = pktalib.SMA(close=close, timeperiod=timeperiod)
@@ -160,6 +176,8 @@ class pktalib:
     @classmethod
     def ATR(self, high, low, close, timeperiod=14):
         try:
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
             return talib.atr(high, low, close, length= timeperiod)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -168,6 +186,8 @@ class pktalib:
     @classmethod
     def TRUERANGE(self, high, low, close):
         try:
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
             return talib.true_range(high, low, close)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -184,7 +204,7 @@ class pktalib:
     @classmethod
     def TriMA(self, close,length=10):
         try:
-            import pandas_ta as talib
+            import pandas_ta_classic as talib
             return talib.trima(close=close, length=length)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -193,7 +213,7 @@ class pktalib:
     @classmethod
     def MACD(self, close, fast, slow, signal):
         try:
-            # import pandas_ta as talib
+            # import pandas_ta_classic as talib
             return talib.macd(close, fast, slow, signal, talib=Imports["talib"])
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -202,6 +222,8 @@ class pktalib:
     @classmethod
     def MFI(self, high, low, close,volume, timeperiod=14):
         try:
+            # Aligning the series
+            # high,low,close,volume = pktalib.align_series(high, low, close,volume, fill_value=0)
             return talib.mfi(high, low, close,volume, length= timeperiod)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -218,6 +240,8 @@ class pktalib:
     @classmethod
     def CCI(self, high, low, close, timeperiod):
         try:
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
             return talib.cci(high, low, close, timeperiod)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -226,6 +250,8 @@ class pktalib:
     @classmethod
     def Aroon(self, high, low, timeperiod):
         try:
+            # Aligning the series
+            # high,low = pktalib.align_series(high, low, fill_value=0)
             return talib.aroon(high, low, timeperiod)
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -240,6 +266,8 @@ class pktalib:
 
     @classmethod
     def STOCHF(self, high, low, close, fastk_period, fastd_period, fastd_matype):
+        # Aligning the series
+        # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
         fastk, fastd = talib.STOCHF(high,
                             low,
                             close,
@@ -270,11 +298,55 @@ class pktalib:
                 close.values, timeperiod, fastk_period, fastd_period, fastd_matype
             )
 
+    @classmethod
+    def highest(self, df,columnName, timeperiod):
+        return df.rolling(timeperiod, min_periods=1)[columnName].max()
+
+    @classmethod
+    def lowest(self, df,columnName, timeperiod):
+        return df.rolling(timeperiod, min_periods=1)[columnName].min()
+    
+    @classmethod
+    def RVM(self, high, low, close, timeperiod):
+        # Aligning the series
+        # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
+        # Relative Volatality Measure
+        #Short-term ATRs
+        short1    = pktalib.ATR(high, low, close,3)
+        short2    = pktalib.ATR(high, low, close,5)
+        short3    = pktalib.ATR(high, low, close,8)
+        shortAvg  = (short1 + short2 + short3) / 3
+
+        #Long-term ATRs
+        long1   = pktalib.ATR(high, low, close,55)
+        long2   = pktalib.ATR(high, low, close,89)
+        long3   = pktalib.ATR(high, low, close,144)
+        longAvg = (long1 + long2 + long3) / 3
+
+        #Combined ATR value
+        combinedATR = (shortAvg + longAvg) / 2
+
+        #Highest and lowest combined ATR over lookback period
+        df_catr = pd.DataFrame(data=combinedATR,columns=["combinedATR"])
+        highestCombinedATR = pktalib.highest(df_catr,"combinedATR", timeperiod)
+        lowestCombinedATR = pktalib.lowest(df_catr,"combinedATR", timeperiod)
+
+        #RVM Calculation
+        diffLowest = (combinedATR - lowestCombinedATR)
+        diffLowest = [x for x in diffLowest if ~np.isnan(x)]
+        diffHighLow = (highestCombinedATR - lowestCombinedATR)
+        diffHighLow = [x for x in diffHighLow if ~np.isnan(x)]
+        df_diff_lowest = pd.DataFrame(data=diffLowest,columns=["diffLowest"])
+        df_diff_highLow = pd.DataFrame(data=diffHighLow,columns=["diffHighLow"])
+        maxHighLow = max(df_diff_highLow["diffHighLow"])
+        rvm =  df_diff_lowest["diffLowest"]/ maxHighLow * 100
+        return round(rvm.tail(1),1)
+
     @classmethod
     def ichimoku(
         self, df, tenkan=None, kijun=None, senkou=None, include_chikou=True, offset=None
     ):
-        import pandas_ta as ta
+        import pandas_ta_classic as ta
 
         ichimokudf, spandf = ta.ichimoku(
             df["high"], df["low"], df["close"], tenkan, kijun, senkou, False, 26
@@ -283,10 +355,10 @@ class pktalib:
 
     @classmethod
     def supertrend(self, df, length=7, multiplier=3):
-        import pandas_ta as ta
+        import pandas_ta_classic as ta
 
         sti = ta.supertrend(
-            df["High"], df["Low"], df["Close"], length=length, multiplier=multiplier
+            df["high"], df["low"], df["close"], length=length, multiplier=multiplier
         )
         # trend, direction, long, short
         # SUPERT_7_3.0  SUPERTd_7_3.0  SUPERTl_7_3.0  SUPERTs_7_3.0
@@ -294,20 +366,177 @@ class pktalib:
 
     @classmethod
     def psar(self, high, low, acceleration=0.02, maximum=0.2):
+        # Aligning the series
+        # high,low = pktalib.align_series(high, low, fill_value=0)
         psar = talib.SAR(high, low, acceleration=acceleration, maximum=maximum)
         return psar
 
+    # @classmethod
+    # def momentum(self, df):
+    #     df.loc[:,'MOM'] = talib.MOM(df.loc[:,"close"],2).apply(lambda x: round(x, 2))
+    #     return df.loc[:,'MOM']
+
+    # @classmethod
+    # def get_dmi_df(self, df):
+    #     df.loc[:,'DMI'] = talib.DX(df.loc[:,"high"],df.loc[:,"low"],df.loc[:,"close"],timeperiod=14)
+    #     return df.loc[:,'DMI']
+
+    # @classmethod
+    # def get_macd_df(self, df):
+    #     df.loc[:,'macd(12)'], df.loc[:,'macdsignal(9)'], df.loc[:,'macdhist(26)'] = talib.MACD(df.loc[:,"close"], fastperiod=12, slowperiod=26, signalperiod=9)
+    #     df.loc[:,'macd(12)'] = df.loc[:,'macd(12)'].apply(lambda x: round(x, 3))
+    #     df.loc[:,'macdsignal(9)']= df.loc[:,'macdsignal(9)'].apply(lambda x: round(x, 3))
+    #     df.loc[:,'macdhist(26)'] = df.loc[:,'macdhist(26)'].apply(lambda x: round(x, 3))
+    #     return df.loc[:,['macd(12)','macdsignal(9)', 'macdhist(26)']]
+
+    # @classmethod
+    # def get_sma_df(self, df):
+    #     df.loc[:,'SMA(10)'] = talib.SMA(df.loc[:,"close"],10).apply(lambda x: round(x, 2))
+    #     df.loc[:,'SMA(50)'] = talib.SMA(df.loc[:,"close"],50).apply(lambda x: round(x, 2))
+    #     return df.loc[:,["close",'SMA(10)', 'SMA(50)']]
+
+    # @classmethod
+    # def get_ema_df(self, df):
+    #     df.loc[:,'EMA(9)'] = talib.EMA(df.loc[:,"close"], timeperiod = 9).apply(lambda x: round(x, 2))
+    #     return df.loc[:,["close",'EMA(9)']]
+
+    # @classmethod
+    # def get_adx_df(self, df):
+    #     df.loc[:,'ADX'] = talib.ADX(df.loc[:,"high"],df.loc[:,"low"], df.loc[:,"close"], timeperiod=14).apply(lambda x: round(x, 2))
+    #     return df.loc[:,'ADX']
+
+    # @classmethod
+    # def get_bbands_df(self, df):
+    #     df.loc[:,'BBands-U'], df.loc[:,'BBands-M'], df.loc[:,'BBands-L'] = talib.BBANDS(df.loc[:,"close"], timeperiod =20)
+    #     df.loc[:,'BBands-U'] = df.loc[:,'BBands-U'].apply(lambda x: round(x, 2))
+    #     df.loc[:,'BBands-M'] = df.loc[:,'BBands-M'].apply(lambda x: round(x, 2))
+    #     df.loc[:,'BBands-L'] = df.loc[:,'BBands-L'].apply(lambda x: round(x, 2))
+    #     return df[["close",'BBands-U','BBands-M','BBands-L']]
+
+    # @classmethod
+    # def get_obv_df(self, df):
+    #     if ("close" not in df.keys()) or ("volume" not in df.keys()):
+    #         return np.nan
+    #     df.loc[:,'OBV'] = talib.OBV(df.loc[:,"close"], df.loc[:,"volume"])
+    #     return df.loc[:,'OBV']
+
+    # @classmethod
+    # def get_atr_df(self, df):
+    #     df.loc[:,'ATR'] = talib.ATR(df.loc[:,"high"], df.loc[:,"low"], df.loc[:,"close"], timeperiod=14).apply(lambda x: round(x, 2))
+    #     return df.loc[:,'ATR']
+
+    # @classmethod
+    # def get_natr_df(self, df):
+    #     df.loc[:,'NATR'] = talib.NATR(df.loc[:,"high"], df.loc[:,"low"], df.loc[:,"close"], timeperiod=14).apply(lambda x: round(x, 2))
+    #     return df.loc[:,'NATR']
+
+    # @classmethod
+    # def get_trange_df(self, df):
+    #     df.loc[:,'TRANGE'] = talib.TRANGE(df.loc[:,"high"], df.loc[:,"low"], df.loc[:,"close"]).apply(lambda x: round(x, 2))
+    #     return df.loc[:,'TRANGE']
+
+    # @classmethod
+    # def get_atr_extreme(self, df):
+    #     """
+    #     ATR Exterme: which is based on 《Volatility-Based Technical Analysis》
+    #     TTI is 'Trading The Invisible'
+
+    #     @return: fasts, slows
+    #     """
+    #     highs = df.loc[:,"high"]
+    #     lows = df.loc[:,"low"]
+    #     closes = df.loc[:,"close"]
+    #     slowPeriod=30
+    #     fastPeriod=3
+    #     atr = self.get_atr_df(df)
+
+    #     highsMean = talib.EMA(highs, 5)
+    #     lowsMean = talib.EMA(lows, 5)
+    #     closesMean = talib.EMA(closes, 5)
+
+    #     atrExtremes = np.where(closes > closesMean,
+    #                 ((highs - highsMean)/closes * 100) * (atr/closes * 100),
+    #                 ((lows - lowsMean)/closes * 100) * (atr/closes * 100)
+    #                 )
+    #     fasts = talib.MA(atrExtremes, fastPeriod)
+    #     slows = talib.EMA(atrExtremes, slowPeriod)
+    #     return fasts, slows, np.std(atrExtremes[-slowPeriod:])
+
+    # @classmethod
+    # def get_atr_ratio(self, df):
+    #     """
+    #     ATR(14)/MA(14)
+    #     """
+    #     closes = df.loc[:,"close"]
+
+    #     atr = self.get_atr_df(df)
+    #     ma = talib.MA(closes, timeperiod=14)
+
+    #     volatility = atr/ma
+
+    #     s = pd.Series(volatility, index=df.index, name='volatility').dropna()
+    #     pd.set_option('mode.chained_assignment', None)
+    #     return pd.DataFrame({'volatility':round(s,2)})
+
+    @classmethod
+    def get_ppsr_df(self, high, low, close,pivotPoint=None):
+        try:
+            # Aligning the series
+            # high,low,close = pktalib.align_series(high, low, close, fill_value=0)
+            PSR = None
+            if pivotPoint is None:
+                return PSR
+            PP = pd.Series((high + low + close) / 3)
+            result = None
+            if pivotPoint != "PP":
+                if pivotPoint == "R1":
+                    result = pd.Series(2 * PP - low)
+                elif pivotPoint == "S1":
+                    result = pd.Series(2 * PP - high)
+                elif pivotPoint == "R2":
+                    result = pd.Series(PP + high - low)
+                elif pivotPoint == "S2":
+                    result = pd.Series(PP - high + low)
+                elif pivotPoint == "R3":
+                    result = pd.Series(high + 2 * (PP - low))
+                elif pivotPoint == "S3":
+                    result = pd.Series(low - 2 * (high - PP))
+            psr = {"close":close, 'PP':round(PP,2)}
+            if pivotPoint != "PP" and result is not None:
+                psr[pivotPoint] = round(result,2)
+            with pd.option_context('mode.chained_assignment', None):
+                PSR = pd.DataFrame(psr)
+        except: # pragma: no cover
+            # default_logger().debug(e, exc_info=True)
+            pass
+        return PSR
+
     @classmethod
     def CDLMORNINGSTAR(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "morningstar")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
             return talib.CDLMORNINGSTAR(open, high, low, close)
 
+    @classmethod
+    def CDLCUPANDHANDLE(self, open, high, low, close):
+        if len(high) < 8:
+            return False
+        return (high.iloc[7] < high.iloc[6] and 
+                high.iloc[7] < high.iloc[5] and 
+                high.iloc[5] < high.iloc[4] and 
+                high.iloc[5] < high.iloc[3] and 
+                high.iloc[3] > high.iloc[2] and 
+                high.iloc[0] > high.iloc[6])
+
     @classmethod
     def CDLMORNINGDOJISTAR(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "morningdojistar")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -316,6 +545,8 @@ class pktalib:
     @classmethod
     def CDLEVENINGSTAR(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "eveningstar")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -324,6 +555,8 @@ class pktalib:
     @classmethod
     def CDLEVENINGDOJISTAR(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "eveningdojistar")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -332,6 +565,8 @@ class pktalib:
     @classmethod
     def CDLLADDERBOTTOM(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "ladderbottom")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -340,6 +575,8 @@ class pktalib:
     @classmethod
     def CDL3LINESTRIKE(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "3linestrike")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -348,6 +585,8 @@ class pktalib:
     @classmethod
     def CDL3BLACKCROWS(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "3blackcrows")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -356,6 +595,8 @@ class pktalib:
     @classmethod
     def CDL3INSIDE(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "3inside")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -364,6 +605,8 @@ class pktalib:
     @classmethod
     def CDL3OUTSIDE(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "3outside")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -372,6 +615,8 @@ class pktalib:
     @classmethod
     def CDL3WHITESOLDIERS(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "3whitesoldiers")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -380,6 +625,8 @@ class pktalib:
     @classmethod
     def CDLHARAMI(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "harami")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -388,6 +635,8 @@ class pktalib:
     @classmethod
     def CDLHARAMICROSS(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "haramicross")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -396,6 +645,8 @@ class pktalib:
     @classmethod
     def CDLMARUBOZU(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "marubozu")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -404,6 +655,8 @@ class pktalib:
     @classmethod
     def CDLHANGINGMAN(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "hangingman")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -412,6 +665,8 @@ class pktalib:
     @classmethod
     def CDLHAMMER(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "hammer")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -420,6 +675,8 @@ class pktalib:
     @classmethod
     def CDLINVERTEDHAMMER(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "invertedhammer")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -428,6 +685,8 @@ class pktalib:
     @classmethod
     def CDLSHOOTINGSTAR(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "shootingstar")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -436,6 +695,8 @@ class pktalib:
     @classmethod
     def CDLDRAGONFLYDOJI(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "dragonflydoji")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -444,6 +705,8 @@ class pktalib:
     @classmethod
     def CDLGRAVESTONEDOJI(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "gravestonedoji")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -452,6 +715,8 @@ class pktalib:
     @classmethod
     def CDLDOJI(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "doji")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
@@ -460,6 +725,8 @@ class pktalib:
     @classmethod
     def CDLENGULFING(self, open, high, low, close):
         try:
+            # Aligning the series
+            # open,high,low,close = pktalib.align_series(open,high, low, close, fill_value=0)
             return talib.cdl_pattern(open, high, low, close, "engulfing")
         except Exception:  # pragma: no cover
             # default_logger().debug(e, exc_info=True)
