@@ -29,7 +29,7 @@ import pandas as pd
 from argparse import Namespace
 from PKDevTools.classes.ColorText import colorText
 from PKDevTools.classes.PKDateUtilities import PKDateUtilities
-from pkscreener.classes import Utility
+from pkscreener.classes import Utility, ImageUtility
 from PKDevTools.classes.log import default_logger
 from pkscreener.classes.ConfigManager import parser, tools
 from pkscreener.classes.Portfolio import Portfolio, PortfolioCollection
@@ -56,7 +56,8 @@ def summariseAllStrategies(testing=False):
                 df_all = pd.concat([df_all, df], axis=0)
             else:
                 df_all = df
-        sys.stdout.write("\x1b[1A")
+        # sys.stdout.write("\x1b[1A")
+        OutputControls().moveCursorUpLines(1)
     if df_all is not None:
         df_all = df_all.replace(np.nan, "-", regex=True)
     return df_all
@@ -231,7 +232,7 @@ def xRaySummary(savedResults=None):
                 continue
             prd_df = df_group[[f"{prd}Pd-%", f"{prd}Pd-10k"]]
             prd_df.loc[:, f"{prd}Pd-10k"] = prd_df.loc[:, f"{prd}Pd-10k"].apply(
-                lambda x: Utility.tools.removeAllColorStyles(x)
+                lambda x: ImageUtility.PKImageTools.removeAllColorStyles(x)
             )
             prd_df = prd_df.replace("-", np.nan, regex=True)
             prd_df = prd_df.replace("", np.nan, regex=True)
@@ -398,7 +399,7 @@ def statScanCalculationForPatterns(*args, **kwargs):
     return scanResults
 
 def ensureColumnsExist(saveResults):
-    columns = ['Stock', 'Date', 'Volume', 'Trend', 'MA-Signal', 'LTP', '52Wk-H',
+    columns = ['Stock', 'Date', "volume", 'Trend', 'MA-Signal', 'LTP', '52Wk-H',
                '52Wk-L', '1-Pd', '2-Pd', '3-Pd', '4-Pd', '5-Pd', '10-Pd', '15-Pd',
                '22-Pd', '30-Pd', 'Consol.', 'Breakout', 'RSI', 'Pattern', 'CCI',
                'LTP1', 'Growth1', 'LTP2', 'Growth2', 'LTP3', 'Growth3', 'LTP4',
@@ -417,12 +418,12 @@ def cleanupData(savedResults):
     saveResults = ensureColumnsExist(saveResults)
     for col in saveResults.columns:
         saveResults.loc[:, col] = saveResults.loc[:, col].apply(
-                lambda x: Utility.tools.removeAllColorStyles(x)
+                lambda x: ImageUtility.PKImageTools.removeAllColorStyles(x)
             )
 
     saveResults["LTP"] = saveResults["LTP"].astype(float).fillna(0.0)
     saveResults["RSI"] = saveResults["RSI"].astype(float).fillna(0.0)
-    saveResults.loc[:, "Volume"] = saveResults.loc[:, "Volume"].apply(
+    saveResults.loc[:, "volume"] = saveResults.loc[:, "volume"].apply(
             lambda x: x.replace("x", "")
         )
     if f"Trend({configManager.daysToLookback}Prds)" not in saveResults.columns:
@@ -437,21 +438,27 @@ def cleanupData(savedResults):
     saveResults.loc[:, f"Consol."] = saveResults.loc[
             :, f"Consol."
         ].apply(lambda x: x.replace("Range:", "").replace("%", ""))
-    saveResults[["Breakout", "Resistance"]] = saveResults[
-            f"Breakout({configManager.daysToLookback}Prds)"
-        ].str.split(" R: ", n=1, expand=True)
-    saveResults.loc[:, "Breakout"] = saveResults.loc[:, "Breakout"].apply(
-            lambda x: x.replace("BO: ", "").replace(" ", "")
-        )
-    saveResults.loc[:, "Resistance"] = saveResults.loc[
-            :, "Resistance"
-        ].apply(lambda x: x.replace("(Potential)", "") if x is not None else x)
-    saveResults["Volume"] = saveResults["Volume"].astype(float).fillna(0.0)
+    try:
+        saveResults = saveResults.loc[:,(saveResults!='').any(axis=0)]
+    except ValueError:
+        # The truth value of a Series is ambiguous.
+        pass
+    if f"Breakout({configManager.daysToLookback}Prds)" in savedResults.columns:
+        saveResults[["Breakout", "Resistance"]] = saveResults[
+                f"Breakout({configManager.daysToLookback}Prds)"
+            ].astype(str).str.split(" R: ", n=1, expand=True)
+        saveResults.loc[:, "Breakout"] = saveResults.loc[:, "Breakout"].apply(
+                lambda x: x.replace("BO: ", "").replace(" ", "")
+            )
+        saveResults.loc[:, "Resistance"] = saveResults.loc[
+                :, "Resistance"
+            ].apply(lambda x: x.replace("(Potential)", "") if x is not None else x)
+        saveResults["Breakout"] = saveResults["Breakout"].astype(float).fillna(0.0)
+        saveResults["Resistance"] = saveResults["Resistance"].astype(float).fillna(0.0)
+    saveResults["volume"] = saveResults["volume"].astype(float).fillna(0.0)
     saveResults[f"Consol."] = (
             saveResults[f"Consol."].astype(float).fillna(0.0)
         )
-    saveResults["Breakout"] = saveResults["Breakout"].astype(float).fillna(0.0)
-    saveResults["Resistance"] = saveResults["Resistance"].astype(float).fillna(0.0)
     saveResults["52Wk-H"] = saveResults["52Wk-H"].astype(float).fillna(0.0)
     saveResults["52Wk-L"] = saveResults["52Wk-L"].astype(float).fillna(0.0)
     saveResults["CCI"] = saveResults["CCI"].astype(float).fillna(0.0)
@@ -471,27 +478,29 @@ def getbacktestPeriod(args):
 
 def statScanCalculations(userArgs, saveResults, periods,progressLabel:str=None):
     scanResults = []
-    task1 = PKTask(f"[{len(saveResults)}] RSI Stats",long_running_fn=statScanCalculationForRSI)
-    task2 = PKTask(f"[{len(saveResults)}] Trend Stats",long_running_fn=statScanCalculationForTrend)
-    task3 = PKTask(f"[{len(saveResults)}] MA Stats",long_running_fn=statScanCalculationForMA)
-    task4 = PKTask(f"[{len(saveResults)}] Volume Stats",long_running_fn=statScanCalculationForVol)
-    task5 = PKTask(f"[{len(saveResults)}] Consolidation Stats",long_running_fn=statScanCalculationForConsol)
-    task6 = PKTask(f"[{len(saveResults)}] Breakout Stats",long_running_fn=statScanCalculationForBO)
-    task7 = PKTask(f"[{len(saveResults)}] 52Week Stats",long_running_fn=statScanCalculationFor52Wk)
-    task8 = PKTask(f"[{len(saveResults)}] CCI Stats",long_running_fn=statScanCalculationForCCI)
-    task9 = PKTask(f"[{len(saveResults)}] CCI Stats",long_running_fn=statScanCalculationForPatterns)
-    task10 = PKTask(f"[{len(saveResults)}] NoFilter Stats",long_running_fn=statScanCalculationForNoFilter)
-    tasksList=[task1,task2,task3,task4,task5,task6,task7,task8,task9,task10]
-    for task in tasksList:
-        task.long_running_fn_args = (userArgs, saveResults, periods, scanResults)
-    if configManager.enablePortfolioCalculations:
-        PKScheduler.scheduleTasks(tasksList,label=progressLabel,showProgressBars=True,timeout=600)
-    else:
+    tasksList = []
+    if saveResults is not None and len(saveResults) >= 1:
+        task1 = PKTask(f"[{len(saveResults)}] RSI Stats",long_running_fn=statScanCalculationForRSI)
+        task2 = PKTask(f"[{len(saveResults)}] Trend Stats",long_running_fn=statScanCalculationForTrend)
+        task3 = PKTask(f"[{len(saveResults)}] MA Stats",long_running_fn=statScanCalculationForMA)
+        task4 = PKTask(f"[{len(saveResults)}] Volume Stats",long_running_fn=statScanCalculationForVol)
+        task5 = PKTask(f"[{len(saveResults)}] Consolidation Stats",long_running_fn=statScanCalculationForConsol)
+        task6 = PKTask(f"[{len(saveResults)}] Breakout Stats",long_running_fn=statScanCalculationForBO)
+        task7 = PKTask(f"[{len(saveResults)}] 52Week Stats",long_running_fn=statScanCalculationFor52Wk)
+        task8 = PKTask(f"[{len(saveResults)}] CCI Stats",long_running_fn=statScanCalculationForCCI)
+        task9 = PKTask(f"[{len(saveResults)}] CCI Stats",long_running_fn=statScanCalculationForPatterns)
+        task10 = PKTask(f"[{len(saveResults)}] NoFilter Stats",long_running_fn=statScanCalculationForNoFilter)
+        tasksList=[task1,task2,task3,task4,task5,task6,task7,task8,task9,task10]
         for task in tasksList:
-            task.long_running_fn(*(task,))
-    for task in tasksList:
-        if task.result is not None and len(task.result) > 0:
-            scanResults.extend(task.result)
+            task.long_running_fn_args = (userArgs, saveResults, periods, scanResults)
+        if configManager.enablePortfolioCalculations:
+            PKScheduler.scheduleTasks(tasksList,label=progressLabel,showProgressBars=True,timeout=600)
+        else:
+            for task in tasksList:
+                task.long_running_fn(*(task,))
+        for task in tasksList:
+            if task.result is not None and len(task.result) > 0:
+                scanResults.extend(task.result)
     return scanResults
 
 def statScanCalculationForCCI(*args, **kwargs):
@@ -825,7 +834,7 @@ def formatGridOutput(df,replacenan=True):
     for col in df.columns:
         try:
             df[col] = df[col].astype(float).fillna(0)
-        except ValueError as e:
+        except ValueError as e: # pragma: no cover
             if len(str(e).split("-")) < 2:
                 # We want to allow value "-" in column values
                 # We don't want to allow '2024-12-03' or 'RSI >=50' or "Trends-50-to-60"
@@ -837,7 +846,7 @@ def formatGridOutput(df,replacenan=True):
             continue
         maxGrowth = df[col].max()
         if "Pd-%" in col:
-            df.loc[:, col] = df.loc[:, col].apply(
+            df.loc[:, col] = df.loc[:, col].astype(str).apply(
                 lambda x: x
                 if (str(x) == "-")
                 else (
@@ -845,7 +854,7 @@ def formatGridOutput(df,replacenan=True):
                         str(x),
                         (
                             (
-                                (colorText.BOLD + colorText.WHITE)
+                                (colorText.WHITE)
                                 if x == maxGrowth
                                 else colorText.GREEN
                             )
@@ -859,7 +868,7 @@ def formatGridOutput(df,replacenan=True):
                 )
             )
         if "Pd-10k" in col:
-            df.loc[:, col] = df.loc[:, col].apply(
+            df.loc[:, col] = df.loc[:, col].astype(str).apply(
                 lambda x: x
                 if (str(x) == "-")
                 else (
@@ -867,7 +876,7 @@ def formatGridOutput(df,replacenan=True):
                         str(x),
                         (
                             (
-                                (colorText.BOLD + colorText.WHITE)
+                                (colorText.WHITE)
                                 if x == maxGrowth
                                 else colorText.GREEN
                             )
@@ -1027,13 +1036,13 @@ def filterMASignalResist(df):
 def filterVolumeLessThan25(df):
     if df is None:
         return None
-    return df[df["Volume"] < 2.5].fillna(0.0)
+    return df[df["volume"] < 2.5].fillna(0.0)
 
 
 def filterVolumeMoreThan25(df):
     if df is None:
         return None
-    return df[df["Volume"] >= 2.5].fillna(0.0)
+    return df[df["volume"] >= 2.5].fillna(0.0)
 
 
 def filterConsolidating10Percent(df):
